#include <unistd.h>
#include <errno.h>
#include "socket.h"
#include "stock.h"

/* read from a socket.
 * read in a loop because fewer bytes might be read because of sockets. 
 * if read returns 0, it indicates that the socket on other side is closed. */

/* since read () and write () are not cancellation points, we wrap these with
 * pthread_testcancel ();
 */

void sock_read (int socket, void *buf, int n) {
   register int bcount=0, br=0;
   
   while (bcount < n) {
      pthread_testcancel ();
      br = read (socket, buf, n-bcount); 
      if (br > 0) {
	 bcount += br;
	 buf += br;
      }
      /* 
       * I have to find out what to do with this...
       * on EINTR, can't continue... for eg. if client shuts down with a 
       * Ctrl-C or just dies, this throws sock_read () into eternal loop
       * 
      else if (errno == EINTR) {
#ifdef DEBUG
	 perror ("sock_read");
#endif
	 continue;
      }
       * 
       */
      else if (br <= 0) {
#ifdef DEBUG
	 perror ("sock_read");
#endif
	 close (socket);
	 kick_player ();
      }
   }
/*   return (bcount); */
}

/* write to a socket.
 * write in a loop to make sure that all the stuff is written. 
 * if EPIPE is set, it indicates that write is performed on close socket. */

void sock_write (int socket, void *buf, int n) {
   register int bcount=0, bw=0;
   
   while (bcount < n) {
      pthread_testcancel ();
      bw = write (socket, buf, n-bcount);
      if (bw >= 0) {
	 bcount += bw;
	 buf += bw;
      }
      else if (errno == EINTR) {
#ifdef DEBUG
	 perror ("sock_write");
#endif
	 continue;
      }
      else if (bw < 0) {
#ifdef DEBUG
	 perror ("sock_write");
#endif
	 close (socket);
	 kick_player ();
      }
   }
/*   return (bcount); */
}

/* These are thread safe versions of the sock_write_int functions 
 * previously they used a static variable for holding temporary value...
 * which is not MT_safe. These are safe :)
 */

inline void sw_s_int (int socket, int16_t s) {
   
   s = htons (s);
   sock_write (socket, &s, sizeof (int16_t));
}

inline void sw_int (int socket, int32_t i) {
   
   i = htonl (i);
   sock_write (socket, &i, sizeof (int32_t));
   
}
